@@ -8,31 +8,25 @@
     }
 
     void assemble_lut18() {
-        std::cerr << "Computing 2^18-entry lookup table...";
         lut18 = (uint8_t*) std::malloc(0x40000); // don't bother freeing; the OS will do that.
         for (uint64_t i = 0; i < 0x40000; i++) {
             lut18[i] = xfl9(i) | (xfl9(i >> 1) << 1) | (xfl9(i >> 2) << 2) | (xfl9(i >> 3) << 3);
         }
-        std::cerr << "done!" << std::endl;
     }
 
     void assemble_lut24() {
         if (lut18 == 0) { assemble_lut18(); }
-        std::cerr << "Computing 2^24-entry lookup table...";
         lut24 = (uint8_t*) std::malloc(0x1000000); // don't bother freeing; the OS will do that.
         for (uint64_t i = 0; i < 0x1000000; i++) {
             lut24[i] = lut18[i & 0x3ffff] | (lut18[i >> 6] << 4);
         }
-        std::cerr << "done!" << std::endl;
     }
 
     void assemble_mix18() {
-        std::cerr << "Computing 2^18-entry mixing table...";
         mix18 = (uint32_t*) std::malloc(0x100000); // see above.
         for (uint64_t i = 0; i < 0x40000; i++) {
             mix18[i] = (i & 0x3f) | (((i >> 4) & 0x3f) << 8) | (((i >> 8) & 0x3f) << 16) | (((i >> 12) & 0x3f) << 24);
         }
-        std::cerr << "done!" << std::endl;
     }
 
     bool iterate_var_leaf(uint64_t *inleaves, uint64_t *outleafx) {
