@@ -3,6 +3,66 @@
 
 namespace apg {
 
+    uint64_t hshear(uint64_t x) {
+
+        uint64_t y = (x & 0xff00ff00ff00ff00ull) | ((x & 0x007f007f007f007full) << 1) | ((x & 0x0080008000800080ull) >> 7);
+        y = (y & 0xffff0000ffff0000ull) | ((y & 0x00003f3f00003f3full) << 2) | ((y & 0x0000c0c00000c0c0ull) >> 6);
+        y = (y & 0xffffffff00000000ull) | ((y & 0x000000000f0f0f0full) << 4) | ((y & 0x00000000f0f0f0f0ull) >> 4);
+        return y;
+
+    }
+
+    uint64_t vshear(uint64_t x) {
+
+        uint64_t y = (x & 0xaaaaaaaaaaaaaaaaull) | ((x & 0x0055555555555555ull) << 8) | ((x & 0x5500000000000000ull) >> 56);
+        y = (y & 0xccccccccccccccccull) | ((y & 0x0000333333333333ull) << 16) | ((y & 0x3333000000000000ull) >> 48);
+        y = (y & 0xf0f0f0f0f0f0f0f0ull) | ((y & 0x000000000f0f0f0full) << 32) | ((y & 0x0f0f0f0f00000000ull) >> 32);
+        return y;
+
+    }
+
+    uint64_t mxor(uint64_t a, uint64_t b) {
+
+        // Shear matrices to obtain:
+        //    c_(i,j) := a_(i,i+j)
+        //    d_(i,j) := b_(i+j,j)
+        uint64_t c = hshear(a);
+        uint64_t d = vshear(b);
+        uint64_t e = c & d;
+
+        // -O3 should unroll this loop:
+        for (int k = 1; k < 8; k++) {
+            c = ((c & 0x7f7f7f7f7f7f7f7full) << 1) | ((c & 0x8080808080808080ull) >> 7);
+            d = (d << 8) | (d >> 56);
+            e ^= (c & d);
+        }
+
+        // e_(i,j) = xor_{k=0}^{7} a_(i,k) b_(k,j)
+        return e;
+
+    }
+
+    uint64_t mor(uint64_t a, uint64_t b) {
+
+        // Shear matrices to obtain:
+        //    c_(i,j) := a_(i,i+j)
+        //    d_(i,j) := b_(i+j,j)
+        uint64_t c = hshear(a);
+        uint64_t d = vshear(b);
+        uint64_t e = c & d;
+
+        // -O3 should unroll this loop:
+        for (int k = 1; k < 8; k++) {
+            c = ((c & 0x7f7f7f7f7f7f7f7full) << 1) | ((c & 0x8080808080808080ull) >> 7);
+            d = (d << 8) | (d >> 56);
+            e |= (c & d);
+        }
+
+        // e_(i,j) = or_{k=0}^{7} a_(i,k) b_(k,j)
+        return e;
+
+    }
+
     uint64_t upset(uint64_t x) {
         uint64_t y = x;
         y |= ((y & 0x5555555555555555) << 1);
