@@ -9,9 +9,92 @@
 #include <utility>
 #include <iostream>
 #include <vector>
+#include <sstream>
+#include <string>
 
 namespace apg {
 
+    struct RleWriter {
+
+        std::ostream &outstream;
+        char terminator;
+        int maxlength;
+        int N;
+        int linelength;
+
+        uint64_t runlength;
+        uint64_t lastx;
+        uint64_t lasty;
+        uint64_t lastc;
+
+        RleWriter(std::ostream &outstream, char terminator, int maxlength, int N) : outstream(outstream),
+            terminator(terminator), maxlength(maxlength), N(N), linelength(0),
+            runlength(0), lastx(0), lasty(0), lastc(0) { }
+
+        void addstring(std::string xs) {
+            if (linelength + ((int) xs.length()) > maxlength) {
+                outstream << terminator;
+                linelength = 0;
+            }
+            outstream << xs;
+            linelength += xs.length();
+        }
+
+        void printrun(uint64_t n, std::string c) {
+            if (n >= 2) {
+                std::ostringstream x;
+                x << n << c;
+                addstring(x.str());
+            } else if (n == 1) {
+                addstring(c);
+            }
+        }
+
+        std::string state2string(uint64_t c) {
+            if (c == 0) {
+                return ((N == 1) ? "b" : ".");
+            } else if (c == 1) {
+                return ((N == 1) ? "o" : "A");
+            } else {
+                uint64_t d = c - 1;
+                std::string a(1, (char) ('A' + (d % 24)));
+                d = d / 24;
+                while (d > 0) {
+                    a = std::string(1, (char) ('p' + ((d + 10) % 11))) + a;
+                    d = d / 11;
+                }
+                return a;
+            }
+        }
+
+        void flushrun() {
+            printrun(runlength, state2string(lastc));
+            runlength = 0;
+        }
+
+        void addcell(uint64_t x, uint64_t y, uint64_t c) {
+
+            if ((y > lasty) || (x > lastx) || (c != lastc)) {
+                flushrun();
+                if (y > lasty) {
+                    printrun(y - lasty, "$");
+                    lasty = y;
+                    lastx = 0;
+                }
+                if (x > lastx) {
+                    printrun(x - lastx, state2string(0));
+                }
+            }
+
+            runlength += 1; lastx = x + 1; lastc = c;
+        }
+
+        void finalise() {
+            flushrun();
+            outstream << "!";
+        }
+    };
+
     // Convert (x, y) pairs into Morton order:
     uint64_t morton32(uint64_t x, uint64_t y) {
         uint64_t z = x | (y << 32);
@@ -204,10 +287,9 @@ namespace apg {
 
         }
 
-        std::vector<std::pair<int64_t, int64_t> > getcells() {
+        std::vector<std::pair<int64_t, int64_t> > getcells() const {
             std::vector<std::pair<int64_t, int64_t> > celllist;
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); ++it ) {
+            for (auto it = world.begin(); it != world.end(); ++it ) {
                 int64_t x = it->first.first;
                 int64_t y = it->first.second;
                 uint64_t tile = it->second;
@@ -524,75 +606,44 @@ namespace apg {
     }
 
     std::string vec2rle(const std::vector<bitworld>& planes, const RuleMapper& rm) {
+
+        std::map<std::pair<int64_t, int64_t>, uint64_t> cells;
+
         int N = planes.size();
-        bitworld allPlanes;
-        for (int i=0; i < N; ++i) {
-            allPlanes += planes[i];
-        }
-        int64_t boundingBox[4];
-        if (!allPlanes.getbbox(boundingBox)) {
-            return "!";
+
+        int64_t minx = 0;
+        int64_t miny = 0;
+
+        for (int i = 0; i < N; i++) {
+            auto celllist = planes[i].getcells();
+            for (auto&& cell : celllist) {
+                cells[std::pair<int64_t, int64_t>(cell.second, cell.first)] += (1ull << i);
+                minx = cell.first;
+                miny = cell.second;
+            }
         }
 
-        const int64_t xmin = boundingBox[0];
-        const int64_t xmax = boundingBox[0] + boundingBox[2];
-        const int64_t ymin = boundingBox[1];
-        const int64_t ymax = boundingBox[1] + boundingBox[3];
+        if (cells.empty()) { return "b!"; }
 
-        int currentColor = 0;
-        int currentRunLength = 0;
-        int sizeAtLastNewline = 0;
-        std::string rle;
-
-        auto encodeColor = [](std::string& rle, int color) {
-            if (color == 0) {
-                rle += '.';
-            } else if (color <= 24) {
-                rle += ('A' + (color-1) % 24);
-            } else {
-                auto it = rle.end();
-                while (color != 0) {
-                    it = rle.insert(it, 'A' + (color-1) % 24);
-                    color /= 24;
-                }
-            }
-        };
-
-        for (int64_t y = ymin; y < ymax; ++y) {
-            for (int64_t x = xmin; x < xmax; ++x) {
-                int colorOfThisCell = 0;
-                for (int i=0; i < N; ++i) {
-                    colorOfThisCell |= planes[i].getcell(x, y) << i;
-                }
-                colorOfThisCell = rm.ll_to_golly(colorOfThisCell);
-                if (rle.size() - sizeAtLastNewline >= 75) {
-                    rle += '\n';
-                    sizeAtLastNewline = rle.size();
-                }
-                if (colorOfThisCell == currentColor) {
-                    ++currentRunLength;
-                } else {
-                    if (currentRunLength != 0) {
-                        if (currentRunLength != 1) {
-                            rle += std::to_string(currentRunLength);
-                        }
-                        encodeColor(rle, currentColor);
-                    }
-                    currentColor = colorOfThisCell;
-                    currentRunLength = 1;
-                }
-            }
-            if (currentRunLength != 0) {
-                if (currentRunLength != 1) {
-                    rle += std::to_string(currentRunLength);
-                }
-                encodeColor(rle, currentColor);
-            }
-            rle += '$';
-            currentRunLength = 0;
+        for (auto it = cells.begin(); it != cells.end(); ++it) {
+            minx = std::min(minx, it->first.second);
+            miny = std::min(miny, it->first.first);
         }
-        rle += '!';
-        return rle;
+
+        std::ostringstream outstream;
+        RleWriter rw(outstream, '\n', 66, N);
+
+        for (auto it = cells.begin(); it != cells.end(); ++it) {
+            uint64_t y = it->first.first - miny;
+            uint64_t x = it->first.second - minx;
+            uint64_t c = rm.ll_to_golly(it->second);
+
+            rw.addcell(x, y, c);
+        }
+
+        rw.finalise();
+
+        return outstream.str();
     }
 
     std::vector<bitworld> rle2vec(std::string rle, const RuleMapper& rm) {
