@@ -4,6 +4,8 @@
 #include <unordered_map>
 #include <set>
 
+#include "dsds.h"
+
 /*
 * This contains code from apgmera, minus the dependence on Golly, for
 * separating and classifying objects.
@@ -37,6 +39,109 @@ namespace apg {
             return (transtable[257] || transtable[68] || b0);
         }
 
+        /**
+         * Better object separation, especially for spaceships:
+         */
+        std::vector<std::string> map_3dsep(const std::vector<std::string> &apgcodes) {
+
+            std::vector<std::string> elements;
+
+            for (auto&& parent : apgcodes) {
+
+                if (parent[0] != 'x') {
+                    elements.push_back(parent);
+                    continue;
+                }
+
+                apg::pattern pat(lab, parent, rule);
+                uint64_t period = pat.ascertain_period();
+
+                if ((pat.empty()) || (period == 0) || (period > 30000)) {
+                    elements.push_back(parent);
+                    continue;
+                }
+
+                pat = pat.shift(262144, 262144);
+
+                int64_t bbox[4] = {0};
+                pat.getrect(bbox);
+
+                if ((bbox[2] > 100000) || (bbox[3] > 100000)) {
+                    elements.push_back(parent);
+                    continue;
+                }
+
+                std::unordered_map<uint64_t, uint64_t> umap;
+                uint64_t id = 0;
+
+                {
+                    apg::pattern pat2 = pat;
+
+                    for (uint64_t t = 0; t <= period; t++) {
+                        std::vector<int64_t> celllist(pat2.totalPopulation() * 2);
+                        pat2.get_coords(&(celllist[0]));
+                        for (uint64_t i = 0; i < celllist.size(); i += 2) {
+                            uint64_t coord = (t << 48) + (celllist[i+1] << 24) + celllist[i];
+                            umap[coord] = id; id += 1;
+                        }
+                        pat2 = pat2[1];
+                    }
+                }
+
+                std::vector<uint64_t> connections;
+
+                connections.push_back((period << 48) + (pat.dy << 24) + pat.dx);
+
+                int range = zoi.size() >> 1;
+
+                for (int i = 0; i <= range*2; i++) {
+                    for (int j = -range*2; j <= range*2; j++) {
+                        int k = (i << 24) + j;
+                        if (k > 0) { connections.push_back(k); }
+                    }
+                }
+
+                for (int i = -range; i <= range; i++) {
+                    for (int j = -range; j <= range; j++) {
+                        connections.push_back((1ull << 48) + (i << 24) + j);
+                    }
+                }
+
+                // disjoint-set data structure:
+                dsds uf(id);
+
+                for (auto it = umap.begin(); it != umap.end(); ++it) {
+                    for (auto&& x : connections) {
+                        uint64_t other = it->first + x;
+                        if (umap.count(other)) { uf.merge(it->second, umap[other]); }
+                    }
+                }
+
+                {
+                    std::vector<apg::pattern> subpats;
+                    std::unordered_map<uint64_t, uint64_t> cmap;
+
+                    for (auto it = umap.begin(); it != umap.end(); ++it) {
+                        if (it->first >= (1ull << 48)) { continue; }
+
+                        uint64_t cid = uf.find(it->second);
+                        if (cmap.count(cid) == 0) {
+                            cmap[cid] = subpats.size();
+                            subpats.emplace_back(lab, "", rule);
+                        }
+
+                        subpats[cmap[cid]] += pat.subrect((it->first & 0xffffff), (it->first >> 24), 1, 1);
+                    }
+
+                    for (auto&& spat : subpats) {
+                        elements.push_back(spat.apgcode());
+                    }
+                }
+            }
+
+            return elements;
+        }
+
         std::vector<std::string> pbbosc(pattern pat, uint64_t n, uint64_t maxn) {
             /*
             * Exhaustively search all partitions of an oscillator into
@@ -478,6 +583,10 @@ namespace apg {
                     // PseudoBangBang not supported for multistate rules; skip separation:
                     elements.push_back(repr);
                 }
+
+                // better separation:
+                elements = map_3dsep(elements);
+
                 decompositions[repr] = elements;
             }
 
