@@ -209,8 +209,6 @@ struct hash_container {
     uint32_cu *d_V;
     uint8_t   *h_V;
     uint32_cu *d_K;
-    uint32_t  *h_seq;
-    uint32_cu *d_seq;
     uint32_cu *interesting;
     uint32_cu *icompressed;
     uint64_t  *h_res;
@@ -221,15 +219,13 @@ struct hash_container {
     uint64_t myriads_per_loop;
     uint64_t hashcount;
 
-    void spin_up(bool verbose, int unicount) {
+    void spin_up(bool verbose) {
 
         laststring = "";
         lastlen = -1;
         myriads_per_loop = 100;
         hashcount = 10000 * myriads_per_loop;
 
-        uint64_t seqlength = unicount;
-
         cudaMalloc((void**)&d_A, hashcount * 64);
         cudaMalloc((void**)&d_F, 64 * myriads_per_loop);
         cudaMallocHost((void**)&h_F, 64 * myriads_per_loop);
@@ -239,10 +235,6 @@ struct hash_container {
         cudaMallocHost((void**)&h_B, hashcount * 32);
         cudaMalloc((void**)&d_K, 256);
         cudaMemcpy(d_K, sha256_k, 256, cudaMemcpyHostToDevice);
-        cudaMalloc((void**)&d_seq, seqlength * 4);
-        cudaMallocHost((void**)&h_seq, seqlength * 4);
-        for (int i = 0; i < seqlength; i++) { h_seq[i] = i; }
-        cudaMemcpy(d_seq, h_seq, seqlength * 4, cudaMemcpyHostToDevice);
         cudaMalloc((void**)&interesting, hashcount * 4);
         cudaMalloc((void**)&icompressed, hashcount / 8);
         cudaMallocHost((void**)&h_res, hashcount / 8);
@@ -266,7 +258,6 @@ struct hash_container {
         cudaFreeHost((void*)h_V);
         cudaFree((void*)d_V);
         cudaFree((void*)d_K);
-        cudaFreeHost((void*)h_seq);
         cudaFree((void*)interesting);
         cudaFree((void*)icompressed);
         cudaFreeHost((void*)h_res);
