@@ -33,35 +33,53 @@ def create_rule(rulestring):
 
     rule_property(rulestring, 'create_rule')
 
-def decode_ltl(rulestring):
+def get_ltl_dict(rulestring):
 
+    for i in range(10):
+        d = str(i)
+        rulestring = rulestring.replace(','+d,'+'+d)
+
+    rulestring = rulestring.replace('-', '..')
     rparts = rulestring.lower().split(',')
-
-    d = {'c': 2, 'm': 1, 'n': 'm'}
-    p = ['r', 'bmin', 'bmax', 'smin', 'smax']
+    d = {'c': 2, 'm': 0, 'n': 'm'}
 
     for r in rparts:
         if (r[0] >= 'a') and (r[0] <= 'z'):
-            if '..' in r[1:]:
-                a, b = r[1:].split('..')
-                d[r[0] + 'min'] = a
-                d[r[0] + 'max'] = b
-            else:
-                d[r[0]] = r[1:]
+            d[r[0]] = r[1:]
         else:
-            d[[x for x in p if x not in d][0]] = r
+            raise ValueError("Invalid LtL-like format")
 
     if d['n'] != 'm':
         raise ValueError("Unsupported LtL neighbourhood type")
 
-    d = {k : int(d[k]) for k in ['r', 'c', 'm', 'smin', 'smax', 'bmin', 'bmax']}
+    for k in 'rcm':
+        d[k] = int(d[k])
 
-    d['smin'] += (1 - d['m'])
-    d['smax'] += (1 - d['m'])
+    for k in 'bs':
+        delta = (1 - d['m']) if (k == 's') else 0
+        things = d[k].split('+')
+        things = [(tuple(x.split('..')) if ('..' in x) else (x, x)) for x in things]
+        d[k] = [(int(a) + delta, int(b) + delta) for (a, b) in things]
+
+    return d
+
+def decode_ltl(rulestring):
+
+    if re.match("^[0-9,]*$", rulestring):
+        # Kellie Evans format:
+        return 'r%sb%st%ss%st%s' % tuple(rulestring.split(','))
+
+    d = get_ltl_dict(rulestring)
+    print("ltl_dict = %s" % str(d))
+
+    if (len(d['b']) == 1) and (len(d['s']) == 1):
+        rulestring = 'r%db%dt%ds%dt%d' % ((d['r'],) + d['b'][0] + d['s'][0])
+    else:
+        raise NotImplementedError("LifeViewer-style HROT notation is currently unsupported")
 
-    rulestring = 'r%db%dt%ds%dt%d' % (d['r'], d['bmin'], d['bmax'], d['smin'], d['smax'])
     if d['c'] > 2:
         rulestring = 'g%d%s' % (d['c'], rulestring)
+
     return rulestring
 
 def sanirule(rulestring, drop_history=False):
