@@ -2,6 +2,16 @@
 import os
 from .rulefiles import rules_dir
 
+def syms(rulestring):
+
+    filename = rulestring if (rulestring[0] == 'x') else ('x' + rulestring)
+    with open(os.path.join(rules_dir, filename + '.h')) as f:
+        for line in f:
+            if 'SYMS="' in line:
+                return line.split('"')[1]
+
+    return "NONE"
+
 def zoi(rulestring):
 
     filename = rulestring if (rulestring[0] == 'x') else ('x' + rulestring)
