@@ -159,43 +159,6 @@ namespace apg {
             }
         }
 
-        void printrun(std::ostream &outstream, uint64_t n, std::string c) {
-            if (n >= 2) {
-                outstream << n << c;
-            } else if (n == 1) {
-                outstream << c;
-            }
-        }
-
-        void printrun(std::ostream &outstream, uint64_t n, std::string c, int &linelength) {
-            std::ostringstream x;
-            printrun(x, n, c);
-            std::string xs = x.str();
-            if (linelength + xs.length() > 66) {
-                outstream << '\n';
-                linelength = 0;
-            }
-            outstream << xs;
-            linelength += xs.length();
-        }
-
-        std::string state2string(uint64_t c) {
-            if (c == 0) {
-                return ((N == 1) ? "b" : ".");
-            } else if (c == 1) {
-                return ((N == 1) ? "o" : "A");
-            } else {
-                uint64_t d = c - 1;
-                std::string a(1, (char) ('A' + (d % 24)));
-                d = d / 24;
-                while (d > 0) {
-                    a = std::string(1, (char) ('p' + ((d + 10) % 11))) + a;
-                    d = d / 11;
-                }
-                return a;
-            }
-        }
-
         void write_rle(std::ostream &outstream, hypernode<I> hnode, std::string rule) {
 
             outstream << "#CLL state-numbering golly" << std::endl;
@@ -207,9 +170,6 @@ namespace apg {
 
             } else {
 
-                auto grm = getMapper(rule);
-                int linelength = 0;
-
                 std::map<std::pair<uint64_t, uint64_t>, uint64_t> cells;
                 getcells_recurse(hnode, 0, 0, cells);
 
@@ -231,33 +191,19 @@ namespace apg {
                 outstream << "x = " << (1 + maxx - minx) << ", y = " << (1 + maxy - miny);
                 outstream << ", rule = " << gollyrule(rule) << std::endl;
 
-                uint64_t lastx = 0; uint64_t lasty = 0; uint64_t lastc = 0;
-                uint64_t runlength = 0;
+                auto grm = getMapper(rule);
+                RleWriter rw(outstream, '\n', 66, N);
 
                 for (auto it = cells.begin(); it != cells.end(); ++it) {
                     uint64_t y = it->first.first - miny;
                     uint64_t x = it->first.second - minx;
                     uint64_t c = grm->ll_to_golly(it->second);
 
-                    if ((y > lasty) || (x > lastx) || (c != lastc)) {
-                        printrun(outstream, runlength, state2string(lastc), linelength);
-                        if (y > lasty) {
-                            printrun(outstream, y - lasty, "$", linelength);
-                            lasty = y;
-                            lastx = 0;
-                        }
-                        if (x > lastx) {
-                            printrun(outstream, x - lastx, state2string(0), linelength);
-                        }
-                        runlength = 0;
-                    }
-                    runlength += 1;
-                    lastx = x + 1;
-                    lastc = c;
+                    rw.addcell(x, y, c);
                 }
 
-                printrun(outstream, runlength, state2string(lastc), linelength);
-                outstream << "!" << std::endl;
+                rw.finalise();
+                outstream << std::endl;
             }
         }
 
