@@ -509,7 +509,7 @@ class Pattern(object):
             if (eventual_oscillator):
                 x = y
 
-        if return_apgcode and allow_guns and self.getrule() == 'b3s23':
+        if allow_guns and self.getrule() == 'b3s23':
             rv = determine_if_gun(self, maxexp)
             if rv is not None:
                 cg, multiple, cw, z, envelope = rv
@@ -1002,4 +1002,6 @@ observer.observe(targetNode, config);
 
         import imageio
         imageio.mimsave(filename, gif, fps=fps)
-        return d['apgcode']
+
+        # return the pattern that was giffed:
+        return y
