@@ -1,4 +1,5 @@
 #pragma once
+#include <memory>
 #include <string>
 #include <stdint.h>
 #include <sstream>
@@ -6,28 +7,21 @@
 namespace apg {
 
     class RuleMapper {
-
         public:
-        virtual uint64_t ll_to_golly(uint64_t orig) = 0;
-        virtual uint64_t golly_to_ll(uint64_t orig) = 0;
+        virtual uint64_t ll_to_golly(uint64_t orig) const = 0;
+        virtual uint64_t golly_to_ll(uint64_t orig) const = 0;
         virtual ~RuleMapper() = default;
-
     };
 
     class IdentityMapper : public RuleMapper {
-
-        public:
-        uint64_t ll_to_golly(uint64_t orig) { return orig; }
-        uint64_t golly_to_ll(uint64_t orig) { return orig; }
-
+        uint64_t ll_to_golly(uint64_t orig) const override { return orig; }
+        uint64_t golly_to_ll(uint64_t orig) const override { return orig; }
     };
 
     class GenerationsMapper : public RuleMapper {
-
         uint64_t n_states;
-        public:
 
-        uint64_t ll_to_golly(uint64_t orig) {
+        uint64_t ll_to_golly(uint64_t orig) const override {
             if ((orig & 3) == 2) {
                 return n_states - ((orig + 2) >> 2);
             } else {
@@ -35,7 +29,7 @@ namespace apg {
             }
         }
 
-        uint64_t golly_to_ll(uint64_t orig) {
+        uint64_t golly_to_ll(uint64_t orig) const override {
             if (orig < 2) {
                 return orig;
             } else {
@@ -43,16 +37,14 @@ namespace apg {
             }
         }
 
-        GenerationsMapper(uint64_t n_states) {
+        public:
+        explicit GenerationsMapper(uint64_t n_states) {
             this->n_states = n_states;
         }
     };
 
     class HistoryMapper : public RuleMapper {
-
-        public:
-
-        uint64_t ll_to_golly(uint64_t orig) {
+        uint64_t ll_to_golly(uint64_t orig) const override {
             if (orig < 3) {
                 return orig;
             } else if (orig == 3) {
@@ -66,30 +58,27 @@ namespace apg {
             }
         }
 
-        uint64_t golly_to_ll(uint64_t orig) {
+        uint64_t golly_to_ll(uint64_t orig) const override {
             return (15 & (0xfab230 >> (orig << 2)));
         }
-
     };
 
-    RuleMapper* getMapper(std::string rule) {
-
-        uint64_t gstates = 0;
+    std::unique_ptr<RuleMapper> getMapper(const std::string& rule) {
         if (rule[0] == 'g') {
+            uint64_t gstates = 0;
             for (uint64_t i = 1; i < rule.size(); i++) {
                 if ((rule[i] < '0') || (rule[i] > '9')) { break; }
                 gstates *= 10;
                 gstates += (rule[i] - '0');
             }
-            return new GenerationsMapper(gstates);
+            return std::unique_ptr<RuleMapper>(new GenerationsMapper(gstates));
         }
 
-        if ((rule.length() > 7) && (rule.substr(rule.length() - 6) == "istory")) {
-            return new HistoryMapper();
+        if ((rule.size() > 7) && (rule.substr(rule.size() - 6) == "istory")) {
+            return std::unique_ptr<RuleMapper>(new HistoryMapper);
         } else {
-            return new IdentityMapper();
+            return std::unique_ptr<RuleMapper>(new IdentityMapper);
         }
-
     }
 
     bool replace(std::string& str, const std::string& from, const std::string& to) {
