@@ -7,6 +7,17 @@
 
 namespace apg {
 
+    void try_copy(std::string &s, char* buffer, int buflen) {
+
+        if (s.length() >= buflen) {
+            std::ostringstream ss;
+            ss << "!" << (((s.length() >> 8) + 1) << 8);
+            s = ss.str();
+        }
+
+        s.copy(buffer, buflen);
+    }
+
     template<typename Out>
     void string_split(const std::string &s, char delim, Out result) {
         std::stringstream ss;
